#!/usr/bin/perl

##### 'RSSReader' script developed by Linux-Development on irc.quakenet.org ( #Linux-Dev ) Version 1.1
##### This program is free software; you can redistribute it and/or modify
##### it under the terms of the GNU General Public License as published by
##### the Free Software Foundation; either version 2 of the License, or
##### (at your option) any later version.
##### This program is distributed in the hope that it will be useful,
##### but WITHOUT ANY WARRANTY; without even the implied warranty of
##### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##### GNU General Public License for more details.
##### You should have received a copy of the GNU General Public License
##### along with this program; if not, write to the Free Software
##### Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

### Xchat registers and hooks.

Xchat::register("irc.quakenet.org:6667 #Linux-Dev RSSReader Script for XChat", "1.1", "This is the /rss Perlscript for XChat", "");
Xchat::print"» Loaded RSSReader Version 1.1 successful from 'Linux-Development' on irc.quakenet.org @ #Linux-Dev - Try \"/rss help\" «";
Xchat::hook_command("rss", "rss_main");

### Perlmodules to load.

use warnings;
use strict;
use XML::RSS;
use LWP::Simple;

### Don't change these variables!

my (@abo, @config, @oldabo);
my ($timer, $off, $onstart, $newtab);
my $xdir = Xchat::get_info('xchatdir');
my $conf = "$xdir/rss.conf";
my $old = "$xdir/rss.old.abo";
my $abo = "$xdir/rss.abo";

### Misc functions on start.

&rss_parse_config('read');
Xchat::hook_timer(60000*$timer, "rss_timer_exec");
if($off == 0 and $onstart == 1){
	rss_timer_exec();
}

### Check if the RSSReader tab is opened.

sub rss_query () {
	my @windows = Xchat::get_list('channels');
	foreach my $temp (@windows){
		if($temp->{'type'} == 3 and $temp->{'channel'} eq '-RSSReader'){
			return 0;
		}
	}
	return -1;
}

### Config parser.

sub rss_parse_config {
	my ($arg) = @_;
	if($arg eq 'read'){
		if(!-e "$conf"){
			open(CREATE,">$conf");
			print CREATE "timer 60\noff 0\nnewtab 1\nonstart 1\n";
			close(CREATE);
		}
		if(!-e "$abo"){
			open(CREATE,">$abo");
			close(CREATE);
		}
		if(!-e "$old"){
			open(CREATE,">$old");
			close(CREATE);
		}
		open(CONFIG,"<$conf");
		@config = (<CONFIG>);
		close(CONFIG);
		open(ABO,"<$abo");
		@abo = (<ABO>);
		close(ABO);
		open(OLDABO,"<$old");
		@oldabo = (<OLDABO>);
		close(OLDABO);
		foreach my $temp (@config){
			chomp($temp);
			my($option, $value) = split /\s+/, $temp;
			if($option eq 'timer'){
				$timer = int($value);
			}
			elsif($option eq 'off' and $value == 1 or $value == 0){
				$off = int($value);
			}
			elsif($option eq 'newtab' and $value == 1 or $value == 0){
				$newtab = int($value);
			}
			elsif($option eq 'onstart' and $value == 1 or $value == 0){
				$onstart = int($value);
			}
		}
	}
	elsif($arg eq 'save'){
		open(NCONF,">$conf");
		print NCONF "timer $timer\noff $off\nnewtab $newtab\nonstart $onstart\n";
		close(NCONF);
	}
	elsif($arg eq 'saveabo'){
		open(NABO,">$abo");
		foreach my $temp (@abo){
			if($temp ne ''){
				print NABO "$temp\n";
			}
		}
		close(NABO);
	}
	elsif($arg eq 'saveold'){
		open(NOLD,">$old");
		foreach my $temp (@oldabo){
			if($temp ne ''){
				print NOLD "$temp\n";
			}
		}
		close(NOLD);
	}
}

### RSSReader main sub.

sub rss_main {
	my $arg = $_[0][1];
	if(!$arg){
		Xchat::print"..:: Usage of RSSReader: ::..";
		Xchat::print"/rss update - Updates the RSS abonements.";
		Xchat::print"/rss list - Lists all added RSS abonements.";
		Xchat::print"/rss switch <option> <value> - Resets the given option.";
		Xchat::print"/rss add <name> <url> - Adds a RSS abonement.";
		Xchat::print"/rss del <name> - Deletes a RSS abonement.";
		Xchat::print"/rss rehash - Rehashs the config files.";
	}
	elsif($arg eq 'rehash'){
		rss_parse_config('read');
		Xchat::print"..:: Rehashed the config files. ::..";
	}
	elsif($arg eq 'update'){
		rss_timer_exec();
	}
	elsif($arg eq 'list'){
		my $count = 0;
		Xchat::print"..:: Added abonements: ::..";
		foreach my $temp (@abo){
			my($name, $url) = split /\s+/, $temp;
			if(defined $name and defined $url){
				$count++;
				Xchat::print"$name - $url";
			}
		}
		Xchat::print"..:: $count RSS abonements are added. ::..";
	}
	elsif($arg eq 'add'){
		my $name = $_[0][2];
		my $url = $_[0][3];
		if(!$name or !$url){
			Xchat::print"..:: Error. One or more of the needed arguments ( name and/or url ) aren't given. ::..";
		} else {
			my $string = "add $name $url";
			my $returnvalue = &rss_check("$string");
			if($returnvalue == 0){
				&rss_operation("$string");
				Xchat::print"..:: I added the following RSS abonement: $name ( $url ). ::..";
			}
			elsif($returnvalue == 1){
				Xchat::print"..:: Error. The name \"$name\" is already used at the RSS abonements list. Please choose another one. ::..";
			}
			elsif($returnvalue == 2){
				Xchat::print"..: Error. The RSS target url \"$url\" is already used in another RSS abonement. Please check this. ::..";
			}
		}
	}
	elsif($arg eq 'del'){
		my $value = $_[0][2];
		my $string = "del $value";
		if(!$value){
			Xchat::print"..:: Usage: /rss del <feed_name>. ::..";
		}
		elsif(rss_check("$string") == 1){
			&rss_operation("$string");
			Xchat::print"..:: I removed the abonement \"$value\". ::..";
		} else {
			Xchat::print"..:: Error. The entered name is not available. ::..";
		}
	}
	elsif($arg eq 'switch'){
		my $option = $_[0][2];
		if(!$option){
			Xchat::print"..:: Available options, their current set and the description: ::..";
			Xchat::print"off     - $off     - Can be 1/0. This will activate or deactivate the RSSReader.";
			Xchat::print"onstart - $onstart - Can be 1/0. Update the RSS abonements on start.";
			Xchat::print"timer   - $timer   - Time in minutes, in which period the RSS abonements will be updated.";
			Xchat::print"newtab  - $newtab  - Can be 1/0. If this options is setted to 1, the RSS updates will be shown in a own tab.";
			Xchat::print"..:: End. ::..";
		}
		elsif($option eq 'off'){
			my $value = $_[0][3];
			if($value == 0 or $value == 1){
				$off = $value;
				Xchat::print"..:: Set \"off\" to $value. ::..";
				rss_parse_config('save');
			} else {
				Xchat::print"..:: Error. The value only can be 0 or 1. ::..";
			}
		}
		elsif($option eq 'newtab'){
			my $value = $_[0][3];
			if($value == 0 or $value == 1){
				$newtab = $value;
				Xchat::print"..:: Set \"newtab\" to $value. ::..";
				rss_parse_config('save');
			} else {
				Xchat::print"..:: Error. The value only can be 0 or 1. ::..";
			}
		}
		elsif($option eq 'onstart'){
			my $value = $_[0][3];
			if($value == 0 or $value == 1){
				$onstart = $value;
				Xchat::print"..:: Set \"onstart\" to $value. ::..";
				rss_parse_config('save');
			} else {
				Xchat::print"..:: Error. The value only can be 0 or 1. ::..";
			}
		}
		elsif($option eq 'timer'){
			my $value = int($_[0][3]);
			if($value > 0){
				if($value != $timer){
					$timer = $value;
					rss_timer_exec('reset');
					Xchat::print"..:: Set \"timer\" to $value minutes. ::..";
					rss_parse_config('save');
				} else {
					Xchat::print"..:: Error. The new timer value is the same as beforde. ::..";
				}
			} else {
				Xchat::print"..:: Error. The value could only be a time in minutes which is higher than 0. ::..";
			}
		} else {
			Xchat::print"..:: $option isn't a valid option. Please have a look at /rss switch. :::.";
		}
	} else {
		rss_main();
	}
}

### Checks availbility of a given item in an array.

sub rss_check {
	my $args = $_[0];
	my($arg, $value, $value2) = split /\s+/, $args;
	if($arg eq 'del'){
		foreach my $temp (@abo){
			chomp($temp);
			my($name, $url) = split /\s+/, $temp;
			if($value eq $name){
				return 1;
			}
		}
		return 0;
	}
	elsif($arg eq 'add'){
		foreach my $temp (@abo){
			chomp($temp);
			my($name, $url) = split /\s+/, $temp;
			if($name eq $value){
				return 1;
			}
			elsif($url eq $value2){
				return 2;
			}
		}
		return 0;
	}
	elsif($arg eq 'old'){
		foreach my $temp (@oldabo){
			chomp($temp);
			if($value eq $temp){
				return 1;
			}
		}
		return 0;
	}
}

### Executes RSSReader operations.

sub rss_operation {
	my $args = $_[0];
	my($arg, $value, $value2) = split /\s+/, $args;
	if($arg eq 'del'){
		foreach my $temp (@abo){
			chomp($temp);
			my($name, $url) = split /\s+/, $temp;
			if($name eq $value){
				@abo = grep{$_ ne $temp} @abo;
				rss_parse_config('saveabo');
			}
		}
	}
	elsif($arg eq 'add'){
		my $string = "$value $value2";
		push(@abo, $string);
		rss_parse_config('saveabo');
	}
}

### Timer execute.

sub rss_timer_exec {
	my ($arg) = @_;
	if($arg and $arg eq 'reset'){
		Xchat::REMOVE;
		Xchat::hook_timer(60000*$timer, "rss_timer_exec");
	}
	elsif($off == 0 or scalar(@abo) == 0){
		my $oldcontext = Xchat::get_context();
		foreach my $temp (@abo){
			chomp($temp);
			my($name, $url) = split /\s+/, $temp;
			my $urltitle = &rss_parse_feed("$url");
			if($urltitle ne '0'){
				my($link, $title) = split /\s+/, $urltitle, 2;
				if($newtab == 1){
					if(&rss_query == -1){
						Xchat::command("QUERY -RSSReader");
					}
					Xchat::set_context("-RSSReader");
				}
				Xchat::print"\002$name\002: $title ( $link )";
			}
			Xchat::set_context("$oldcontext");
			rss_parse_config('saveold');
		}
	}
	Xchat::KEEP;
}

### XML::RSS parser.

sub rss_parse_feed {
	my $feed = $_[0];
	my $rss = new XML::RSS;
	my $content = get("$feed");
	if($content){
		$rss->parse($content);
		my $item = (@{$rss->{'items'}}[0]);
		my $url = $item->{'link'};
		my $urlcheck = "old $url";
		if(rss_check("$urlcheck") == 0){
			my $string = "$url $item->{'title'}";
			push(@oldabo, $url);
			return $string;
		}
	}
	return 0;
}

#####
##### That's all, folks!
#####